@@ -145,6 +145,7 @@ class GitCommand(Resource):
     # Resource
     def render(self, request):
         self.request = request
+        log.msg("Spawning git command: %s with args: %r" % (self.cmd, self.args))
         reactor.spawnProcess(self, self.cmd, self.args)
 
         return NOT_DONE_YET
@@ -239,7 +240,9 @@ class InfoRefs(Resource):
                 return "Invalid RPC: " + request.args['service'][0]
 
             rpc = request.args['service'][0][4:]
-            request.write(git_packet('# service=git-' + rpc) + git_packet())
+            request.setHeader('Content-Type',
+                              'application/x-git-' + rpc + '-advertisement')
+            request.write(git_packet('# service=git-' + rpc + '\n') + git_packet())
             cmd = self.gitcommand
             args = [os.path.basename(cmd), rpc, '--stateless-rpc',
                     '--advertise-refs', self.gitpath]
@@ -309,13 +312,15 @@ class GitResource(Resource):
         # /info/refs
         if (len(pathparts) >= 2 and
             pathparts[-2] == 'info' and
-            pathparts[-1] == 'refs'):
+            pathparts[-1] == 'refs' and
+            path_info['repository_fs_path'] is not None):
             writerequired = ('service' in request.args and
                              request.args['service'][0] == 'git-receive-pack')
             resource = InfoRefs(path_info['repository_fs_path'])
 
         # /git-upload-pack (client pull)
-        elif len(pathparts) >= 1 and pathparts[-1] == 'git-upload-pack':
+        elif (len(pathparts) >= 1 and pathparts[-1] == 'git-upload-pack' and
+              path_info['repository_fs_path'] is not None):
             cmd = 'git'
             args = [os.path.basename(cmd), 'upload-pack', '--stateless-rpc',
                     path_info['repository_fs_path']]
@@ -324,7 +329,8 @@ class GitResource(Resource):
                               'application/x-git-upload-pack-result')
 
         # /git-receive-pack (client push)
-        elif len(pathparts) >= 1 and pathparts[-1] == 'git-receive-pack':
+        elif (len(pathparts) >= 1 and pathparts[-1] == 'git-receive-pack' and
+              path_info['repository_fs_path'] is not None):
             writerequired = True
             cmd = 'git'
             args = [os.path.basename(cmd), 'receive-pack',
@@ -344,7 +350,7 @@ class GitResource(Resource):
                     headers = get_headers()
                     break
 
-            if filename is not None:
+            if filename is not None and path_info['repository_fs_path'] is not None:
                 for key, val in headers.items():
                     request.setHeader(key, val)
 
@@ -400,7 +406,7 @@ class GitResource(Resource):
         return ForbiddenResource()
 
 
-class GitHTMLRealm(object):
+class GitHTTPRealm(object):
     implements(IRealm)
 
     def __init__(self, authnz, git_configuration,
@@ -426,10 +432,10 @@ def create_factory(authnz, git_configuration, git_viewer=None):
     if git_viewer is None:
         git_viewer = NoResource()
     elif not IResource.providedBy(git_viewer):
-        raise ValueError("git_viewer should be either implement IResource")
+        raise ValueError("git_viewer should implement IResource")
 
     credentialFactories = [BasicCredentialFactory('Git Repositories')]
-    gitportal = Portal(GitHTMLRealm(authnz, git_configuration,
+    gitportal = Portal(GitHTTPRealm(authnz, git_configuration,
                                     credentialFactories, git_viewer))
 
     if hasattr(authnz, 'check_password'):
